
import '../types/types.dart';
import '../systems/system.dart';
import '../forwarders/effect_forwarder.dart';

class EffectSystem<State, Event> {

  EffectSystem._raw(this._system);

  final System<State, Event> _system;

  EffectSystem.pure(EffectSystemRun<State, Event> run):
    this._raw(System.pure(_toSystemRun(run)));

  Dispose run({
    Effect<State, Event>? effect,
  }) => _system.run(
    effect: effect
  );

  EffectSystem.create({
    required State initialState,
    required Reduce<State, Event> reduce,
  }): this._raw(System<State, Event>
    .create(initialState: initialState)
    .add(reduce: reduce));  

  EffectSystem<State, Event> forward({
    required CopySystem<State, Event> copy,
  }) {
    final _next = copy(this._system);
    return EffectSystem<State, Event>._raw(_next);
  }

  EffectSystem<State, Event> copy(
    CopyEffectSystemRun<State, Event> copy
  ) {
    final CopySystem<State, Event> _copySystem = _toCopySystem(copy);
    return forward(copy: _copySystem);
  }

  EffectSystem<State, Event> runWithContext<Context>({
    required Context Function() createContext,
    required Dispose Function(Context context, EffectSystemRun<State, Event> run, Effect<State, Event>? nextEffect) run,
  }) => forward(copy: (system) => system.runWithContext<Context>(
    createContext: createContext,
    run: (context, systemRun, nextReduce, nextEffect) {
      final _effectSystemRun = _toEffectSystemRun(systemRun);
      return run(context, _effectSystemRun, nextEffect);
    },
  ));

  EffectSystem<State, Event> withContext<Context>({
    required Context Function() createContext,
    required ContextEffect<Context, State, Event> effect,
    void Function(Context context)? dispose,
  }) => forward(copy: (system) => system.withContext(
    createContext: createContext,
    effect: effect,
    dispose: dispose,
  ));

  EffectSystem<State, Event> add({
    required Effect<State, Event> effect
  }) => forward(copy: (system) => system.add(
    effect: effect,
  ));
}

extension AsEffectSystem<State, Event> on System<State, Event> {
  EffectSystem<State, Event> asEffectSystem() => EffectSystem._raw(this);
}

extension AsSystem<State, Event> on EffectForwarder<State, Event> {
  EffectSystem<State, Event> asEffectSystem() => EffectSystem.pure(({effect}) {
    if (effect == null) return Dispose.nothing();
    return add(effect: effect);
  });
}

CopySystem<State, Event> _toCopySystem<State, Event>(
  CopyEffectSystemRun<State, Event> copy
) => (system) => system.copy((run) {
  final _run = _toEffectSystemRun(run);
  final _next = copy(_run);
  return ({reduce, effect}) => _next(effect: effect); 
});

EffectSystemRun<State, Event> _toEffectSystemRun<State, Event>(
  Run<State, Event> run
) => ({effect}) => run(effect: effect);

Run<State, Event> _toSystemRun<State, Event>(
  EffectSystemRun<State, Event> effectSystemFun
) => ({reduce, effect}) => effectSystemFun(effect: effect);