import 'package:love/love.dart';

import 'effect_system.dart';
import '../systems/react_on_system.dart';
import '../types/types.dart';

extension EffectSystemReactOperators<State, Event> on EffectSystem<State, Event> {

  EffectSystem<State, Event> reactState({
    required void Function(State state, State? oldState, Dispatch<Event> dispatch) effect,
  }) => forward(copy: (system) => system.reactState(
    effect: effect
  ));

  EffectSystem<State, Event> react<Value>({
    required Value Function(State state) value,
    AreEqual<Value>? areEqual,
    bool skipFirstValue = false,
    required void Function(Value value, Dispatch<Event> dispatch) effect,
  }) => forward(copy: (system) => system.react(
    value: value,
    areEqual: areEqual,
    skipFirstValue: skipFirstValue,
    effect: effect,
  ));

  EffectSystem<State, Event> reactLatest<Value>({
    required Value Function(State state) value,
    AreEqual<Value>? areEqual,
    bool skipFirstValue = false,
    required Dispose? Function(Value value, Dispatch<Event> dispatch) effect,
  }) => forward(copy: (system) => system.reactLatest(
    value: value,
    areEqual: areEqual,
    skipFirstValue: skipFirstValue,
    effect: effect,
  ));

  EffectSystem<State, Event> reactRequest<Request>({
    required Request? Function(State state) request,
    AreEqual<Request>? areEqual,
    bool skipFirstRequest = false,
    required void Function(Request request, Dispatch<Event> dispatch) effect,
  }) => forward(copy: (system) => system.reactRequest(
    request: request,
    areEqual: areEqual,
    skipFirstRequest: skipFirstRequest,
    effect: effect,
  ));

  EffectSystem<State, Event> reactLatestRequest<Request>({
    required Request? Function(State state) request,
    AreEqual<Request>? areEqual,
    bool skipFirstRequest = false,
    required Dispose? Function(Request request, Dispatch<Event> dispatch) effect,
  }) => forward(copy: (system) => system.reactLatestRequest(
    request: request,
    areEqual: areEqual,
    skipFirstRequest: skipFirstRequest,
    effect: effect,
  ));
}