

import 'effect_system.dart';
import '../forwarders/effect_forwarder.dart';
import '../types/types.dart';
import '../systems/system.dart';

extension EffectSystemShareOperators<State, Event> on EffectSystem<State, Event> {

  EffectSystem<State, Event> share() => copy((run) {
    final _forwarder = EffectForwarder<State, Event>();
    int _count = 0;
    Dispose? _sourceDispose;
    return ({effect}) {
      final nextEffect = effect ?? (_, __, ___, ____) {};
      final Dispose _dispose = _forwarder.add(effect: nextEffect);
      _count += 1;

      if (_count == 1) {
        _sourceDispose = run(effect: _forwarder.effect);
      }

      return Dispose(() {
        _dispose();
        _count -= 1;
        final sourceDispose = _sourceDispose;
        if (_count == 0 && sourceDispose != null) {
          _sourceDispose = null;
          sourceDispose();
        }
      });
   };
  });

  EffectSystem<State, Event> shareForever() => copy((run) {
    final _forwarder = EffectForwarder<State, Event>();
    bool _running = false; 

    return ({effect}) {

      final nextEffect = effect ?? (_, __, ___, ____) {};
      final Dispose _dispose = _forwarder.add(effect: nextEffect);

      if (!_running) {
        _running = true;
        run(effect: _forwarder.effect);
      }

      return _dispose;
    };
  });
}


extension ShareSystem<State, Event> on System<State, Event> {
  EffectSystem<State, Event> share() => this.asEffectSystem()
    .share();
      
  EffectSystem<State, Event> shareForever() => this.asEffectSystem()
    .shareForever();
}