
import 'package:love/src/types/latest_context.dart';

import 'system.dart';
import '../types/types.dart';

extension OnOperators<State, Event> on System<State, Event> {

  System<State, Event> onEvent({
    required void Function(State state, Event? event, Dispatch<Event> dispatch) effect
  }) => add(
    effect: (state, oldState, event, dispatch) {
      effect(state, event, dispatch);
    }
  );

  System<State, Event> on<ChildEvent>({
    ChildEvent? Function(Event event)? test,
    State Function(State state, ChildEvent event)? reduce,
    void Function(State state, ChildEvent event, Dispatch<Event> dispatch)? effect,
  }) {
    final _test = test ?? _safeAs;
    return add(
      reduce: reduce == null ? null : (state, event) {
        return _testEvent(event,
          test: _test,
          then: (ChildEvent childEvent) => reduce(state, childEvent),
        ) ?? state;
      }, 
      effect: effect == null ? null : (state, oldState, event, dispatch) {
        if (oldState != null && event != null) {
          _testEvent(event,
            test: _test,
            then: (ChildEvent childEvent) => effect(state, childEvent, dispatch),
          );
        } 
      }
    );
  }

  System<State, Event> onLatest<ChildEvent>({
    ChildEvent? Function(Event event)? test,
    required Dispose? Function(State state, ChildEvent event, Dispatch<Event> dispatch) effect,
  }){
    final _test = test ?? _safeAs;
    return withContext<LatestContext<Event>>(
      createContext: () => LatestContext(),
      effect: (context, state, oldState, event, dispatch) {
        if (event != null) {
          _testEvent(event,
            test: _test,
            then: (ChildEvent childEvent) {
              context.disposePreviousEffect();
              context.dispose = effect(state, childEvent, context.versioned(dispatch));  
            },
          );
        }
      },
      dispose: (context) => context.disposePreviousEffect(),
    );
  } 

  System<State, Event> onRun({
    required void Function(State initialState, Dispatch<Event> dispatch) effect
  }) => add(effect: (state, oldState, event, dispatch) {
    if (event == null) {
      effect(state, dispatch);
    }
  });

  System<State, Event> onDispose({ 
    required void Function() run 
  }) => copy((_run) => ({reduce, effect}) {
      final dispose = Dispose(run);
      final sourceDispose = _run(reduce: reduce, effect: effect);
      return sourceDispose.append(dispose);
    },
  );
}

Result? _testEvent<Result, ChildEvent, Event>(Event event, {
  required ChildEvent? test(Event event),
  required Result then(ChildEvent childEvent),
}) {
  final childEvent = test(event);
  if (childEvent != null) return then(childEvent);
}

R? _safeAs<T, R>(T value) => value is R ? value as R : null;
