import 'dart:async';
import 'system.dart';
import '../types/types.dart';

extension RxOperators<State, Event> on System<State, Event> {
  
  System<State, Event> effects(
    Stream<Event> Function(Stream<Moment<State, Event>> moments) effects
  ) => withContext<_Context<State, Event>>(
    createContext: () => _Context(
      StreamController(),
    ),
    effect: (context, state, oldState, event, dispatch) {
      if (event == null) {
        final _events = effects(context.moments.stream);
        context.subscription = _events.listen(dispatch.call);
      }
      context.moments.add(Moment(state, oldState, event));      
    },
    dispose: (context) {
      context.subscription?.cancel();
      context.subscription = null;
      context.moments.close();
    },
  );
}

class _Context<State, Event> {
  _Context(this.moments);
  final StreamController<Moment<State, Event>> moments;
  StreamSubscription<Event>? subscription;
}