import '../types/types.dart';
import '../utils.dart';

typedef CopySystem<State, Event> = System<State, Event> Function(System<State, Event> system);

class System<State, Event> {

  System.pure(this._run);

  final Run<State, Event> _run;

  Dispose run({ 
    Reduce<State, Event>? reduce,
    Effect<State, Event>? effect,
  }) {
    var _isDisposed = false;
    final _dispose = _run(
      reduce: reduce, 
      effect: effect,
    );
    return Dispose(() {
      if (_isDisposed) return;
      _isDisposed = true;
      _dispose();
    });
  }

  System.create({
    required State initialState,
  }): this.pure(_create(initialState: initialState));

  System<State, Event> copy(
    CopyRun<State, Event> copy
  ) {
    final _next = copy(this._run);
    return System.pure(_next);
  }

  System<State, Event> runWithContext<Context>({
    required Context Function() createContext,
    required Dispose Function(Context context, Run<State, Event> run, Reduce<State, Event>? nextReduce, Effect<State, Event>? nextEffect) run,
  }) {
    final _run = run;
    return copy((run) => ({reduce, effect}) {
      final context = createContext();
      return _run(context, run, reduce, effect);
    });
  }

  System<State, Event> withContext<Context>({
    required Context Function() createContext,
    Reduce<State, Event>? reduce,
    ContextEffect<Context, State, Event>? effect,
    void Function(Context context)? dispose,
  }) => runWithContext<Context>(
    createContext: createContext,
    run: (context, run, nextReduce, nextEffect) {
      final Effect<State, Event>? _effect = effect == null ? null : (state, oldState, event, dispatch) {
        effect(context, state, oldState, event, dispatch);
      };
      final sourceDispose = run(
        reduce: combineReduce(reduce, nextReduce),
        effect: combineEffect(_effect, nextEffect),
      );
      final Dispose combinedDispose = dispose == null ? sourceDispose : Dispose(() {
        dispose(context);
        sourceDispose();
      });
      return combinedDispose;
    }
  );

  System<State, Event> add({
    Reduce<State, Event>? reduce,
    Effect<State, Event>? effect,
  }) => withContext<Null>(
    createContext: () => null,
    reduce: reduce,
    effect: effect == null ? null : (context, state, oldState, event, dispatch) 
      => effect(state, oldState, event, dispatch),
  );
}

Run<State, Event> _create<State, Event>({
  required State initialState,
}) => ({reduce, effect}) {
  assert(reduce != null, 'reduce is nil when system run!');
  if (reduce == null) return Dispose.nothing();

  State? _state;
  bool _isDisposed = false;
  bool _comsuming = false;

  late Consume<Event> _consume;
  
  void __dispatch(Event event) {
    if (_isDisposed) return;
    if (!_comsuming) {
      _consume(event);
    } else {
      Future(() => __dispatch(event));
    }
  }

  final _dispatch = Dispatch(__dispatch);

  _consume = (Event? event) {
    _comsuming = true;
    final oldState = _state;
    final state = oldState != null && event != null 
      ? reduce(oldState, event)
      : initialState;
    _state = state;
    effect?.call(state, oldState, event, _dispatch);
    _comsuming = false;
  };

  _consume(null); // initial event

  return Dispose(() {
    if (_isDisposed) return;
    _isDisposed = true;
  });
};