

typedef AreEqual<T> = bool Function(T it1, T it2);

typedef Reduce<State, Event> = State Function(State state, Event event);

typedef DispatchFunc<Event> = void Function(Event event);
class Dispatch<Event> {
  Dispatch(this._func);
  final DispatchFunc<Event> _func;
  void call(Event event) => _func(event);
}

typedef Consume<Event> = void Function(Event? event);

typedef DisposeFunc = void Function();
class Dispose {
  Dispose(this._func);
  final DisposeFunc _func;
  void call() => _func();

  Dispose.nothing(): this(() {});

  Dispose.merge(List<Dispose> disposes): this(() {
    for (final dispose in disposes) dispose();
  });

  Dispose insert(Dispose other) => Dispose(() {
    other.call();
    this.call();
  });
  
  Dispose append(Dispose other) => Dispose(() {
    this.call();
    other.call();
  });
}

typedef Effect<State, Event> = void Function(State state, State? oldState, Event? event, Dispatch<Event> dispatch);
typedef ContextEffect<Context, State, Event> = void Function(Context context, State state, State? oldState, Event? event, Dispatch<Event> dispatch);

// class Love<State, Event> {
//   Love({this.reduce, this.effect});
//   final Reduce<State, Event>? reduce;
//   final Effect<State, Event>? effect;
// }
typedef Run<State, Event> = Dispose Function({
  Reduce<State, Event>? reduce,
  Effect<State, Event>? effect,
});
typedef CopyRun<State, Event> = Run<State, Event> Function(Run<State, Event> run);

typedef EffectSystemRun<State, Event> = Dispose Function({Effect<State, Event>? effect});
typedef CopyEffectSystemRun<State, Event> = EffectSystemRun<State, Event> Function(EffectSystemRun<State, Event> sourceSystemFun);

class Moment<State, Event> {
  final State state;
  final State? oldState;
  final Event? event;
  const Moment(this.state, this.oldState, this.event);
}