
import 'package:test/test.dart';

import '../test_utils.dart';

void main() {
  
  test('EffectSystem.add.effect', () async {

    int _invoked = 0;
    List<String> _stateParameters = [];
    List<String?> _oldStateParameters = [];
    List<String?> _eventParameters = [];

    final it = await testEffectSystem<String, String>(
      system: createTestEffectSystem(initialState: 'a')
        .add(
          effect: (state, oldState, event, dispatch) {
            if (event == 'c') {
              dispatch('i');
            }
            _stateParameters.add(state);
            _oldStateParameters.add(oldState);
            _eventParameters.add(event);
            _invoked += 1;
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'c'),
        dispatch(20, 'd'),
        dispatch(30, 'e'),
        dispose(40),
        dispatch(50, 'f'),
      ],
      awaitMilliseconds: 60,
    );

    expect(it.events, [
      null,
      'b',
      'c',
      'i',
      'd',
      'e',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|d',
      'a|b|c|i|d|e',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|d',
    ]);

    expect(it.isDisposed, true);

    expect(_stateParameters, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|d',
      'a|b|c|i|d|e',
    ]);

    expect(_oldStateParameters, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|d',
    ]);

    expect(_eventParameters, [
      null,
      'b',
      'c',
      'i',
      'd',
      'e',
    ]);

    expect(_invoked, 6);

  });

  test('EffectSystem.add.effect.order', () async {

    int _invoked1 = 0;
    List<String> _stateParameters1 = [];
    List<String?> _oldStateParameters1 = [];
    List<String?> _eventParameters1 = [];

    int _invoked2 = 0;
    List<String> _stateParameters2 = [];
    List<String?> _oldStateParameters2 = [];
    List<String?> _eventParameters2 = [];

    List<int> _orders = [];

    final it = await testEffectSystem<String, String>(
      system: createTestEffectSystem(initialState: 'a')
        .add(
          effect: (state, oldState, event, dispatch) {
            if (event == 'c') {
              dispatch('i');
            }
            _stateParameters1.add(state);
            _oldStateParameters1.add(oldState);
            _eventParameters1.add(event);
            _invoked1 += 1;
            _orders.add(1);
          },
        ).add(
          effect: (state, oldState, event, dispatch) {
            if (event == 'c') {
              dispatch('j');
            }
            _stateParameters2.add(state);
            _oldStateParameters2.add(oldState);
            _eventParameters2.add(event);
            _invoked2 += 1;
            _orders.add(2);
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'c'),
        dispatch(20, 'd'),
        dispatch(30, 'e'),
        dispose(40),
        dispatch(50, 'f'),
      ],
      awaitMilliseconds: 60,
    );

    expect(it.events, [
      null,
      'b',
      'c',
      'i',
      'j',
      'd',
      'e',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|j',
      'a|b|c|i|j|d',
      'a|b|c|i|j|d|e',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|j',
      'a|b|c|i|j|d',
    ]);

    expect(it.isDisposed, true);

    expect(_stateParameters1, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|j',
      'a|b|c|i|j|d',
      'a|b|c|i|j|d|e',
    ]);

    expect(_oldStateParameters1, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|j',
      'a|b|c|i|j|d',
    ]);

    expect(_eventParameters1, [
      null,
      'b',
      'c',
      'i',
      'j',
      'd',
      'e',
    ]);

    expect(_invoked1, 7);

    expect(_stateParameters2, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|j',
      'a|b|c|i|j|d',
      'a|b|c|i|j|d|e',
    ]);

    expect(_oldStateParameters2, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|j',
      'a|b|c|i|j|d',
    ]);

    expect(_eventParameters2, [
      null,
      'b',
      'c',
      'i',
      'j',
      'd',
      'e',
    ]);

    expect(_invoked2, 7);

    expect(_orders, [
      1, 2,
      1, 2,
      1, 2,
      1, 2,
      1, 2,
      1, 2,
      1, 2,
    ]);

  });
}