
import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {
  
  test('EffectSystem.effects', () async {

    int _invoked = 0;
    List<String> _stateParameters = [];
    List<String?> _oldStateParameters = [];
    List<String?> _eventParameters = [];

    final it = await testEffectSystem<String, String>(
      system: createTestEffectSystem(initialState: 'a')
        .effects((moments) => moments.asyncExpand((moment) {
          _stateParameters.add(moment.state);
          _oldStateParameters.add(moment.oldState);
          _eventParameters.add(moment.event);
          _invoked += 1;
          if (moment.event == 'c') {
            return Stream.value('i');
          };
          return null;
        })),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'c'),
        dispatch(20, 'd'),
        dispatch(30, 'e'),
        dispose(40),
        dispatch(50, 'f'),
      ],
      awaitMilliseconds: 60,
    );


    expect(it.events, [
      null,
      'b',
      'c',
      'i',
      'd',
      'e',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|d',
      'a|b|c|i|d|e',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|d',
    ]);

    expect(it.isDisposed, true);

    expect(_eventParameters, [
      null,
      'b',
      'c',
      'i',
      'd',
      'e',
    ]);

    expect(_stateParameters, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|d',
      'a|b|c|i|d|e',
    ]);

    expect(_oldStateParameters, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|i',
      'a|b|c|i|d',
    ]);

    expect(_invoked, 6);
  });
}