import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {
  
  test('EffectSystem.onLatest', () async {

    int _testInvoked = 0;
    int _effectVersion = 0;
    List<String> _eventParameters = [];
    List<String> _reqeustParameters = [];
    List<int> _disposedVersions = [];

    final it = await testEffectSystem<String, String>(
      system: createTestEffectSystem(initialState: 'a')
        .onLatest<String>(
          test: (event) {
            _eventParameters.add(event);
            _testInvoked += 1;
            return event == 'b' ? 'i' : null;
          },
          effect: (state, request, dispatch) {
            _reqeustParameters.add(request);
            _effectVersion += 1;
            delayed(1, () => dispatch(request));
            final version = _effectVersion;
            return Dispose(() {
              _disposedVersions.add(version);
            });
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'c'),
        dispatch(10, 'a'),
        dispatch(20, 'b'),
        dispatch(20, 'c'),
        dispatch(20, 'a'),
        dispatch(20, 'b'),
        dispatch(20, 'c'),
        dispose(30),
        dispatch(40, 'a'),
        dispatch(50, 'b'),
      ],
      awaitMilliseconds: 60,
    );

    expect(it.events, [
      null,
      'b',
      'i',
      'c',
      'a',
      'b',
      'c',
      'a',
      'b',
      'c',
      'i',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|a',
      'a|b|i|c|a|b|c|a|b',
      'a|b|i|c|a|b|c|a|b|c',
      'a|b|i|c|a|b|c|a|b|c|i',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|a',
      'a|b|i|c|a|b|c|a|b',
      'a|b|i|c|a|b|c|a|b|c',
    ]);

    expect(it.isDisposed, true);

    expect(_eventParameters, [
      'b',
      'i',
      'c',
      'a',
      'b',
      'c',
      'a',
      'b',
      'c',
      'i',
    ]);

    expect(_reqeustParameters, [
      'i',
      'i',
      'i',
    ]);

    expect(_testInvoked, 10);
    expect(_effectVersion, 3);
    expect(_disposedVersions, [1, 2, 3]);
  });

}