import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {

  test('EffectSystem.reactLatest', () async {

    int _valueInvoked = 0;
    int _effectInvoked = 0;
    int _effectVersion = 0;
    List<String> _stateParameters = [];
    List<String> _valueParameters = [];
    List<int> _disposedVersions = [];

    final it = await testEffectSystem<String, String>(
      system: createTestEffectSystem(initialState: 'a')
        .reactLatest<String>(
          value: (state) {
            _stateParameters.add(state);
            _valueInvoked += 1;
            return state.substring(state.length - 1);
          },
          effect: (value, dispatch) {
            _valueParameters.add(value);
            _effectInvoked += 1;
            if (value == 'b') {
              _effectVersion += 1;
              delayed(1, () => dispatch('i'));
              final version = _effectVersion;
              return Dispose(() {
                _disposedVersions.add(version);
              });
            }
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'c'),
        dispatch(10, 'a'),
        dispatch(20, 'b'),
        dispatch(20, 'c'),
        dispose(30),
        dispatch(40, 'a'),
        dispatch(50, 'b'),
      ],
      awaitMilliseconds: 60,
    );

    expect(it.events, [
      null,
      'b',
      'i',
      'c',
      'a',
      'b',
      'c',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
    ]);

    expect(it.isDisposed, true);

    expect(_stateParameters, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
    ]);

    expect(_valueParameters, [
      'a',
      'b',
      'i',
      'c',
      'a',
      'b',
      'c',
    ]);
    
    expect(_valueInvoked, 7);
    expect(_effectInvoked, 7);
    expect(_effectVersion, 2);

    expect(_disposedVersions, [1, 2]);
  });

}