import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {

  test('EffectSystem.reactRequest', () async {

    int _invoked = 0;
    int _dispatchInvoked = 0;
    List<String> _stateParameters = [];
    List<String> _requestParameters = [];

    final it = await testEffectSystem<String, String>(
      system: createTestEffectSystem(initialState: 'a')
        .reactRequest<String>(
          request: (state) {
            _stateParameters.add(state);
            _invoked += 1;
            return state.endsWith('b') ? 'i' : null;
          },
          effect: (request, dispatch) {
            _requestParameters.add(request);
            dispatch(request);
            _dispatchInvoked += 1;
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'c'),
        dispatch(10, 'a'),
        dispatch(20, 'b'),
        dispatch(20, 'c'),
        dispose(30),
        dispatch(40, 'a'),
        dispatch(50, 'b'),
      ],
      awaitMilliseconds: 60,
    );

    expect(it.events, [
      null,
      'b',
      'i',
      'c',
      'a',
      'b',
      'c',
      'i',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|i',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
    ]);

    expect(it.isDisposed, true);

    expect(_stateParameters, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|i',
    ]);

    expect(_requestParameters, [
      'i',
      'i',
    ]);

    expect(_invoked, 8);
    expect(_dispatchInvoked, 2);
  });

}