
import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {
 
  test('EffectSystem.shareForever', () async {

    int _disposeInvoked = 0;
    int _disposeInvoked1 = 0;
    int _disposeInvoked2 = 0;
    List<String> _stateParameters = [];
    List<String?> _oldStatesParameters = [];
    List<String?> _eventParameters = [];
    List<String?> _eventParameters1 = [];
    List<String?> _eventParameters2 = [];

    Dispose? dispose1;
    Dispose? dispose2;
    
    final system = createTestSystem(initialState: 'a')
      .withContext<Null>(
        createContext: () => null,
        effect: (_, state, oldState, event, dispatch) {
          _stateParameters.add(state);
          _oldStatesParameters.add(oldState);
          _eventParameters.add(event);
          if (event == null) {
            delayed(0, () => dispatch('b'));
            delayed(10, () => dispatch('c'));
            delayed(40, () => dispatch('a'));
            delayed(50, () => dispatch('b'));
            delayed(60, () => dispatch('c'));
            delayed(90, () => dispatch('a'));
            delayed(100, () => dispatch('b'));
            delayed(110, () => dispatch('c'));
          }
        },
        dispose: (_) {
          _disposeInvoked += 1;
        },
      )
      .shareForever();

    delayed(20, () => dispose2?.call());
    delayed(70, () => dispose1?.call());
    
    dispose1 = () {
      final _dispose1 = system
        .run(effect: (state, oldState, event, dispatch) {
          _eventParameters1.add(event);
        });
      return Dispose(() {
        _disposeInvoked1 += 1;
        _dispose1();
      });
    }(); 

    
    dispose2 = () {
      final _dispose2 = system
        .run(effect: (state, oldState, event, dispatch) {
          _eventParameters2.add(event);
        });
      return Dispose(() {
        _disposeInvoked2 += 1;
        _dispose2();
      });
    }();

    await delayed<Null>(30);

    expect(_disposeInvoked, 0);
    expect(_disposeInvoked1, 0);
    expect(_disposeInvoked2, 1);

    await delayed<Null>(120 - 30);

    expect(_eventParameters, [
      null,
      'b',
      'c',
      'a',
      'b',
      'c',
      'a',
      'b',
      'c',
    ]);

    expect(_stateParameters, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|a',
      'a|b|c|a|b',
      'a|b|c|a|b|c',
      'a|b|c|a|b|c|a',
      'a|b|c|a|b|c|a|b',
      'a|b|c|a|b|c|a|b|c',
    ]);

    expect(_oldStatesParameters, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|a',
      'a|b|c|a|b',
      'a|b|c|a|b|c',
      'a|b|c|a|b|c|a',
      'a|b|c|a|b|c|a|b',
    ]);

    expect(_eventParameters1, [
      null,
      'b',
      'c',
      'a',
      'b',
      'c',
    ]);

    expect(_eventParameters2, [
      null,
      'b',
      'c',
    ]);

    expect(_disposeInvoked, 0);
    expect(_disposeInvoked1, 1);
    expect(_disposeInvoked2, 1);
  });
}