
import 'package:test/test.dart';
import 'package:love/love.dart';

void main() {

  test('EffectForwarder.effect', () {

    List<String> _states = [];
    List<String?> _oldStates = [];
    List<String?> _events = [];
    List<String> _dispatchedEvents = [];
    int _invoked = 0;

    final forwarder = EffectForwarder<String, String>();

    final Effect<String, String> effect = (state, oldState, event, dispatch) {
      _states.add(state);
      _oldStates.add(oldState);
      _events.add(event);
      dispatch('e');
      _invoked += 1;
    }; 
    
    final dispatch = Dispatch<String>((event) {
      _dispatchedEvents.add(event);
    });

    final dispose = forwarder.add(effect: effect);

    forwarder.effect('a', null, null, dispatch);

    expect(_states, ['a']);
    expect(_oldStates, [null]);
    expect(_events, [null]);
    expect(_dispatchedEvents, ['e']);
    expect(_invoked, 1);

    dispose();
  });

  test('EffectForwarder.dispose', () {

    List<String> _states = [];
    List<String?> _oldStates = [];
    List<String?> _events = [];
    List<String> _dispatchedEvents = [];
    int _invoked = 0;

    final forwarder = EffectForwarder<String, String>();

    final Effect<String, String> effect = (state, oldState, event, dispatch) {
      _states.add(state);
      _oldStates.add(oldState);
      _events.add(event);
      dispatch('e');
      _invoked += 1;
    }; 

    final dispatch = Dispatch<String>((event) {
      _dispatchedEvents.add(event);
    });

    final dispose = forwarder.add(effect: effect);

    forwarder.effect('a', null, null, dispatch);

    expect(_states, ['a']);
    expect(_oldStates, [null]);
    expect(_events, [null]);
    expect(_dispatchedEvents, ['e']);
    expect(_invoked, 1);

    dispose();
   
    forwarder.effect('a|b', 'a', 'b', Dispatch((_) {})); 

    expect(_states, ['a']);
    expect(_oldStates, [null]);
    expect(_events, [null]);
    expect(_dispatchedEvents, ['e']);
    expect(_invoked, 1);
  });


  test('EffectForwarder.replay.empty', () {

    List<String> _states = [];
    List<String?> _oldStates = [];
    List<String?> _events = [];
    int _invoked = 0;

    final forwarder = EffectForwarder<String, String>();

    final Effect<String, String> effect = (state, oldState, event, dispatch) {
      _states.add(state);
      _oldStates.add(oldState);
      _events.add(event);
      _invoked += 1;
    }; 

    final dispose = forwarder.add(effect: effect);

    expect(_states, <String>[]);
    expect(_oldStates, <String?>[]);
    expect(_events, <String?>[]);
    expect(_invoked, 0);

    dispose();
  });

  test('EffectForwarder.replay.last', () {

    List<String> _states = [];
    List<String?> _oldStates = [];
    List<String?> _events = [];
    List<String> _dispatchedEvents = [];
    int _invoked = 0;

    final Effect<String, String> effect = (state, oldState, event, dispatch) {
      _states.add(state);
      _oldStates.add(oldState);
      _events.add(event);
      dispatch('e');
      _invoked += 1;
    }; 

    final dispatch = Dispatch<String>((event) {
      _dispatchedEvents.add(event);
    });

    final forwarder = EffectForwarder<String, String>();
    forwarder.effect('a', 'b', 'b', dispatch);

    final dispose = forwarder.add(effect: effect);

    expect(_states, ['a']);
    expect(_oldStates, [null]);
    expect(_events, [null]);
    expect(_dispatchedEvents, ['e']);
    expect(_invoked, 1);

    dispose();
  });

  test('EffectForwarder.forward.order', () {

    List<int> _orders = [];

    final forwarder = EffectForwarder<String, String>();

    final Effect<String, String> effect1 = (state, oldState, event, dispatch) {
      _orders.add(1);
    }; 

    final Effect<String, String> effect2 = (state, oldState, event, dispatch) {
      _orders.add(2);
    }; 

    final Effect<String, String> effect3 = (state, oldState, event, dispatch) {
      _orders.add(3);
    }; 

    final dispatch = Dispatch<String>((_) {});

    final dispose1 = forwarder.add(effect: effect1);
    final dispose2 = forwarder.add(effect: effect2);
    final dispose3 = forwarder.add(effect: effect3);

    forwarder.effect('a', null, null, dispatch);

    expect(_orders, [1, 2, 3]);

    dispose2();

    forwarder.effect('a', null, null, dispatch);

    expect(_orders, [1, 2, 3, 1, 3]);

    dispose1();
    dispose3();
  });

}