
import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {

  test('System.onEvent', () async {
    
    int _invoked = 0;
    int _dispatchInvoked = 0;
    List<String> _stateParameters = [];
    List<String?> _eventParameters = [];

    final it = await testSystem<String, String>(
      system: createTestSystem(initialState: 'a')
        .onEvent(
          effect: (state, event, dispatch) {
            if (event == 'b') {
              dispatch('c');
              _dispatchInvoked += 1;
            }
            _stateParameters.add(state);
            _eventParameters.add(event);
            _invoked += 1;
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'a'),
        dispatch(20, 'b'),
        dispose(30),
        dispatch(40, 'a'),
        dispatch(50, 'b'),
      ],
      awaitMilliseconds: 60,
    );

    expect(it.events, [
      null,
      'b',
      'c',
      'a',
      'b',
      'c',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|a',
      'a|b|c|a|b',
      'a|b|c|a|b|c',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|a',
      'a|b|c|a|b',
    ]);

    expect(_stateParameters, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|a',
      'a|b|c|a|b',
      'a|b|c|a|b|c',
    ]);

    expect(_eventParameters, [
      null,
      'b',
      'c',
      'a',
      'b',
      'c',
    ]);

    expect(_invoked, 6);
    expect(_dispatchInvoked, 2);
  });
}