
import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {

  test('System.on', () async {

    int _invoked = 0;
    List<String> _stateParameters = [];
    List<String> _eventParameters = [];

    final it = await testSystem<String, String>(
      system: createTestSystem(initialState: 'a')
        .on<String>(
          test: (event) => event == 'b' ? 'i' : null,
          reduce: (state, event) => '$state|$event',
          effect: (state, event, dispatch) {
            _stateParameters.add(state);
            _eventParameters.add(event);
            _invoked += 1;
            dispatch('c');
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'a'),
        dispatch(20, 'b'),
        dispose(30),
        dispatch(40, 'f'),
      ],
      awaitMilliseconds: 50,
    );

    expect(it.events, [
      null,
      'b',
      'c',
      'a',
      'b',
      'c',
    ]);

    expect(it.states, [
      'a',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b|i',
      'a|b|i|c|a|b|i|c',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b|i',
    ]);

    expect(it.isDisposed, true);

    expect(_stateParameters, [
      'a|b|i',
      'a|b|i|c|a|b|i',
    ]);

    expect(_eventParameters, [
      'i',
      'i',
    ]);

    expect(_invoked, 2);
  
  });
}