
import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {
  
  test('System.reactLatestRequest', () async {

    int _requestInvoked = 0;
    int _effectVersion = 0;
    List<String> _stateParameters = [];
    List<String> _valueParameters = [];
    List<int> _disposedVersions = [];

    final it = await testSystem<String, String>(
      system: createTestSystem(initialState: 'a')
        .reactLatestRequest<String>(
          request: (state) {
            _stateParameters.add(state);
            _requestInvoked += 1;
            return state.endsWith('b') ? 'i' : null;
          },
          effect: (value, dispatch) {
            _valueParameters.add(value);
            _effectVersion += 1;
            delayed(1, () => dispatch(value)); 
            final version = _effectVersion;
            return Dispose(() {
              _disposedVersions.add(version);
            });
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'c'),
        dispatch(10, 'a'),
        dispatch(20, 'b'),
        dispatch(20, 'c'),
        dispatch(20, 'a'),
        dispatch(20, 'b'),
        dispatch(20, 'c'),
        dispose(30),
        dispatch(40, 'a'),
        dispatch(50, 'b'),
      ],
      awaitMilliseconds: 60,
    );

    expect(it.events, [
      null,
      'b',
      'i',
      'c',
      'a',
      'b',
      'c',
      'a',
      'b',
      'c',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|a',
      'a|b|i|c|a|b|c|a|b',
      'a|b|i|c|a|b|c|a|b|c',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|a',
      'a|b|i|c|a|b|c|a|b',
    ]);

    expect(it.isDisposed, true);

    expect(_stateParameters, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|a',
      'a|b|i|c|a|b|c|a|b',
      'a|b|i|c|a|b|c|a|b|c',
    ]);

    expect(_valueParameters, [
      'i',
      'i',
      'i',
    ]);

    expect(_requestInvoked, 10);
    expect(_effectVersion, 3);
    expect(_disposedVersions, [1, 2, 3]);
  });

}