
import 'package:test/test.dart';
import 'package:love/love.dart';
import '../test_utils.dart';

void main() {

  test('System.react', () async {

    int _valueInvoked = 0;
    int _effectInvoked = 0;
    int _dispatchInvoked = 0;
    List<String> _stateParameters = [];
    List<String> _valueParameters = [];

    final it = await testSystem<String, String>(
      system: createTestSystem(initialState: 'a')
        .react<String>(
          value: (state) {
            _stateParameters.add(state);
            _valueInvoked += 1;
            return state.substring(state.length - 1);
          },
          effect: (value, dispatch) {
            _valueParameters.add(value);
            _effectInvoked += 1;
            if (value == 'b') {
              dispatch('i');
              _dispatchInvoked += 1;
            }
          },
        ),
      events: (dispatch, dispose) => [
        dispatch(0, 'b'),
        dispatch(10, 'c'),
        dispatch(10, 'a'),
        dispatch(20, 'b'),
        dispatch(20, 'c'),
        dispose(30),
        dispatch(40, 'a'),
        dispatch(50, 'b'),
      ],
      awaitMilliseconds: 60,
    );

    expect(it.events, [
      null,
      'b',
      'i',
      'c',
      'a',
      'b',
      'c',
      'i',
    ]);

    expect(it.states, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|i',
    ]);

    expect(it.oldStates, [
      null,
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
    ]);

    expect(it.isDisposed, true);

    expect(_stateParameters, [
      'a',
      'a|b',
      'a|b|i',
      'a|b|i|c',
      'a|b|i|c|a',
      'a|b|i|c|a|b',
      'a|b|i|c|a|b|c',
      'a|b|i|c|a|b|c|i',

    ]);

    expect(_valueParameters, [
      'a',
      'b',
      'i',
      'c',
      'a',
      'b',
      'c',
      'i',
    ]);

    expect(_valueInvoked, 8);
    expect(_effectInvoked, 8);
    expect(_dispatchInvoked, 2);
  });
}