
import 'package:test/test.dart';
import 'package:love/love.dart';

import '../test_utils.dart';

void main() {

  test('System.run', () async {

    Dispose? dispose;

    List<String> _states = [];
    List<String?> _oldStates = [];
    List<String?> _events = [];
    bool _isDisposed = false;

    final system = createTestSystem(initialState: 'a');
    
    final _dispose = system.run(
      effect: (state, oldState, event, dispatch) {
        _states.add(state);
        _oldStates.add(oldState);
        _events.add(event);
        if (event == null) {
          delayed(0, () => dispatch('b'));
          delayed(10, () => dispatch('c'));
          delayed(20, () => dispatch('d'));
          delayed(30, () => dispatch('e'));
          delayed(40, () => dispose?.call());
          delayed(50, () => dispatch('f'));
        }
      },
    );

    dispose = Dispose(() {
      _isDisposed = true;
      _dispose();
    });

    await delayed<Null>(60);
  
    expect(_events, [
      null,
      'b',
      'c',
      'd',
      'e',
    ]);

    expect(_states, [
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|d',
      'a|b|c|d|e',
    ]);

    expect(_oldStates, [
      null,
      'a',
      'a|b',
      'a|b|c',
      'a|b|c|d',
    ]);

    expect(_isDisposed, true);
    
  });
}